unit wp.Controls.ToggleSwitch;

{$mode Delphi}

interface

uses
  Classes, SysUtils, Controls, Graphics, ExtCtrls,
  Math, FPImage, Types, IntfGraphics;

type

  { TToggleSwitch }

  TToggleSwitch = class(TCustomControl)
    strict private
      FImages: array[0..3] of TCustomBitmap;
      FRadius: Integer;
      FMargin: Integer;
      FSliderRange: Integer;
      FSliderPos: Integer;
      FSpeed: Integer;
      FChecked: Boolean;
      FRoll: Boolean;
      FTimer: TTimer;
    private
      procedure SetChecked(AValue: Boolean);
      procedure SetRoll(AValue: Boolean);
      procedure SetSpeed(AValue: Integer);
{
      function GetWidth: Integer;
      procedure SetWidth(AValue: Integer); reintroduce;
      function GetHeight: Integer;
      procedure SetHeight(AValue: Integer); reintroduce;
}
      procedure TimerAnimation(Sender: TObject);
    protected
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      procedure Paint; override;
      procedure Update; reintroduce;
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
//      property Width: Integer read GetWidth write SetWidth;
//      property Height: Integer read GetHeight write SetHeight;
      property Checked: Boolean read FChecked write SetChecked default False;
      property Roll: Boolean read FRoll write SetRoll default True;
      property Speed: Integer read FSpeed write SetSpeed default 10;
  end;

implementation

{$R imgtoggleswitch.res}

function RotatePoint(const APoint: TPoint; AAngle: Double): TPoint;
var
  sa, ca: Double;
begin
  SinCos(AAngle, sa, ca);
  Result.X := Round( ca * APoint.X + sa * APoint.Y);
  Result.Y := Round(-sa * APoint.X + ca * APoint.Y);
end;

procedure RotateImage(Img: TFPCustomImage; Angle: Double);
var
  Buffer: TFPCustomImage;
  x, y: Integer;
  C, P: TPoint;
begin
  Buffer := TFPMemoryImage.Create(Img.Width, Img.Height);
  C := Point(Img.Width div 2, Img.Height div 2);
  for y := 0 to Buffer.Height-1 do
    for x := 0 to Buffer.Width-1 do
    begin
      P := RotatePoint(Point(x, y) - C, Angle) + C;
      if (P.X >= 0) and (P.Y >= 0) and (P.X < Img.Width) and (P.Y < Img.Height) then
        Buffer.Colors[x, y] := Img.Colors[P.X, P.Y]
      else
        Buffer.Colors[x, y] := colTransparent;
    end;
  for y := 0 to Img.Height-1 do
    for x := 0 to Img.Width-1 do
      Img.Colors[x, y] := Buffer[x, y];
  Buffer.Free;
end;

procedure BlendImages(Img1, Img2: TFPCustomImage; AFactor: Double);
var
  x, y: Integer;
  r,g,b,a: Word;
  f1, f2: Double;
begin
  f1 := 1.0 - AFactor;
  f2 := AFactor;
  for y := 0 to Img1.Height-1 do
    for x := 0 to Img1.Width-1 do
    begin
      r := round(f1 * Img1.Colors[x, y].Red   + f2 * Img2.Colors[x, y].Red);
      g := round(f1 * Img1.Colors[x, y].Green + f2 * Img2.Colors[x, y].Green);
      b := round(f1 * Img1.Colors[x, y].Blue  + f2 * Img2.Colors[x, y].Blue);
      a := round(f1 * Img1.Colors[x, y].Alpha + f2 * Img2.Colors[x, y].Alpha);
      Img1.Colors[x, y] := FPColor(r, g, b, a);
    end;
end;

{ TToggleSwitch }

constructor TToggleSwitch.Create(AOwner: TComponent);
var
  i: Integer;
begin
  inherited Create(AOwner);
  Self.Parent := TWinControl(AOwner);
  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FTimer.Interval := 10;
  FTimer.OnTimer := TimerAnimation;
  for i := Low(FImages) to High(FImages) do
    FImages[i] := TPortableNetworkGraphic.Create;
  try
    FImages[0].LoadFromResourceName(HInstance,'TS_RED');
    FImages[1].LoadFromResourceName(HInstance,'TS_GREEN');
    FImages[2].LoadFromResourceName(HInstance,'TS_EXIT');
    FImages[3].LoadFromResourceName(HInstance,'TS_OK');
  finally
  end;
  FMargin := (FImages[0].Height - FImages[2].Height) div 2;
  FRadius := FImages[2].Height div 2;
  FSliderRange := FImages[0].Width - 2*(FMargin + FRadius);
  FSliderPos := 0;
  FRoll := True;
  Self.Width := FImages[0].Width;
  Self.Height := FImages[0].Height;
  Self.AutoSize := True;
  Self.Caption := 'Test';
  Self.Visible := True;
  FChecked := False;
  Self.Checked := FChecked;
end;

destructor TToggleSwitch.Destroy;
var
  i: Integer;
begin
  FTimer.Free;
  for i := Low(FImages) to High(FImages) do
    FImages[i].Free;
  inherited Destroy;
end;

procedure TToggleSwitch.SetChecked(AValue: Boolean);
begin
  if (FChecked = AValue) then
    Exit;
  FChecked := AValue;
  FTimer.Enabled := True;
  Self.Invalidate;
end;

procedure TToggleSwitch.SetRoll(AValue: Boolean);
begin
  if (FRoll = AValue) then
    Exit;
  FRoll := AValue;
  Self.Invalidate;
end;

procedure TToggleSwitch.SetSpeed(AValue: Integer);
begin
  if ((AValue < 10) or (AValue > 50) or (FSpeed = AValue)) then
    Exit;
  FSpeed := AValue;
  FTimer.Interval := FSpeed;
end;

(*
function TToggleSwitch.GetWidth: Integer;
begin
  Result := FImages[0].Width;
end;

procedure TToggleSwitch.SetWidth(AValue: Integer);
begin
//  Self.Width := FImages[0].Width;
end;

function TToggleSwitch.GetHeight: Integer;
begin
  Result := FImages[0].Height;
end;

procedure TToggleSwitch.SetHeight(AValue: Integer);
begin
//  Self.Height := FImages[0].Height;
end;
*)

procedure TToggleSwitch.TimerAnimation(Sender: TObject);
begin
  if (not FChecked) then
    begin
      Inc(FSliderPos, 2);
      if (FSliderPos >= FSliderRange) then
      begin
        FSliderPos := FSliderRange;
        FTimer.Enabled := False;
      end;
    end
  else
    begin
      Dec(FSliderPos, 2);
      if (FSliderPos <= 0) then
      begin
        FSliderPos := 0;
        FTimer.Enabled := False;
      end;
    end;
  Self.Update;
end;

procedure TToggleSwitch.Update;
begin
  inherited Update;
  Self.Invalidate;
end;

procedure TToggleSwitch.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var
  R: TRect;
begin
  inherited MouseUp(Button, Shift, X, Y);
  R := ClientRect;
  if PtInRect(R, Point(X, Y)) then
    begin
      FChecked := (not FChecked);
      FTimer.Enabled := True;
      Self.Invalidate;
    end;
end;

procedure TToggleSwitch.Paint;
const
  TWO_PI = 2.0 * pi;
var
  imgEXIT, imgOK, imgRed, imgGreen: TLazIntfImage;
  bmp: TBitmap;
  f: Double;
begin
  if Self.Width <> FImages[0].Width then
    Self.Width := FImages[0].Width;
  if Self.Height <> FImages[0].Height then
    Self.Height := FImages[0].Height;

  f := FSliderPos / FSliderRange;

  bmp := TBitmap.Create;
  try
    imgRed := FImages[0].CreateIntfImage;
    imgGreen := FImages[1].CreateIntfImage;
    BlendImages(imgGreen, imgRed, 1.0 - f);
    bmp.Pixelformat := pf32Bit;
    bmp.Assign(imgGreen);
    Self.Canvas.Draw(0, 0, bmp);
  finally
    bmp.Free;
    imgRed.Free;
    imgGreen.Free;
  end;

  imgExit := FImages[2].CreateIntfImage;
  imgOK := FImages[3].CreateIntfImage;
  bmp := TBitmap.Create;
  try
    BlendImages(imgOK, imgEXIT, 1.0 - f);
    if FRoll then
      RotateImage(imgOK, TWO_PI * f);

    bmp.Pixelformat := pf32Bit;
    bmp.Assign(imgOK);

    Self.Canvas.Draw(FMargin + FSliderPos, FMargin, bmp);
  finally
    bmp.Free;
    imgOK.Free;
    imgExit.Free;
  end;
  inherited Paint;
end;


end.

